import java.beans.*;
import java.io.*;

public class A {
	/**
	 * 他の種類のオブジェクトへの参照
	 */
	private B refB;
	private C[] refCary;
	/**
	 * 名前
	 */
	private String name;

	/**
	 * 無引数のコンストラクタはXMLEnc/Decに必須
	 */
	public A() {
	}

	public A(String n) {
		name = n;
	}

	public void sayName() {
		System.out.println(name);
		if (refB != null)
			refB.sayName();
		if (refCary != null)
			for (int i = 0; i < refCary.length; i++)
				refCary[i].sayName();
	}

	public B getRefB() {
		return refB;
	}

	public void setRefB(B refB) {
		this.refB = refB;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	/**
	 * XMLEncoderをつかってオブジェクトを永続化． もし，refBにもオブジェクトがあればそれも同時に永続化．
	 * 
	 * @param fname
	 *            永続先のファイル名
	 */
	public void save(String fname) {
		try {
			XMLEncoder enc = new XMLEncoder(new BufferedOutputStream(
					new FileOutputStream(fname)));
			enc.writeObject(this);
			enc.close();
		} catch (Exception ec) {
			ec.printStackTrace();
		}
		System.out.println("** save " + this + " into " + fname);
	}

	/**
	 * XMLDecoderを使って永続化オブジェクトをロード． もし，refBのオブジェクトも一緒に永続化されていればそれも復元．
	 * 
	 * @param fname
	 *            ファイル名
	 * @return 復元されたクラスAへの参照
	 */
	public static A load(String fname) {
		A retA = null;
		try {
			XMLDecoder dec = new XMLDecoder(new BufferedInputStream(
					new FileInputStream(fname)));
			retA = (A) dec.readObject();
			dec.close();
		} catch (Exception e) {
		}
		System.out.println("** load " + retA + " from " + fname);
		return retA;
	}

	public C[] getRefCary() {
		return refCary;
	}

	public void setRefCary(C[] refCary) {
		this.refCary = refCary;
	}
}
